var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Translator class
 */
var Translator = function () {

  /**
   * Initialiazes the Translator
   *
   * @param {Object} config
   */
  function Translator(config) {
    _classCallCheck(this, Translator);

    var defaultConfig = {
      source: 'en',
      missingPlaceholder: '⚠️',
      debug: false, // TODO: Make this available only in development mode
      translations: {}
    };

    this.config = defaultConfig;

    // this.activeLanguage = null;

    this.translations = {};
    /*
    if (typeof this.config !== 'undefined') {
      let key = null;
      for (key in config) {
        if (config.hasOwnProperty(key)) {
          this.config[object] = config[object]
        }
      }
    } else {
      this.config = config;
    }*/
  }

  _createClass(Translator, [{
    key: 't',
    value: function t(message) {
      var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};


      if (typeof message !== 'string') {
        return this.error('Wrong usage of translate()');
      }

      var defaultConfig = {
        source: 'en',
        missingPlaceholder: '⚠️',
        debug: false, // TODO: Make this available only in `development` mode
        translations: {}
      };

      // default config
      if (typeof this.config === 'undefined') {
        this.config = defaultConfig;
      }

      // Used for an app creator to verify that he didn't forget some hardcoded text
      if (this.config.debug) {
        return this.debugMode(message);
      }

      // If a single string or number is passed, convert it to an array
      // We're expecting a single numerical placeholder
      if (typeof data === 'string' || typeof data === 'number') {
        data = [data];
      }

      if (typeof this.translations[message] !== 'undefined' && this.translations[message] !== '') {
        message = this.translations[message];
      }

      // Substiture plurals
      message = this.pluralize(message, data);

      // Placeholders detection pa
      var placeholdersReg = /({([^}]+)})/g;

      var retval = [];

      var ongoing = '';
      var match = false;
      var lastIndex = 0;

      while (match = placeholdersReg.exec(message)) {
        ongoing += message.substring(lastIndex, match.index);
        if (typeof data[match[2]] === 'undefined') {
          ongoing += this.config.missingPlaceholder;
        } else if (_typeof(data[match[2]]) === 'object') {
          // retval.push(React.createElement(Text, null, ongoing));
          retval.push(data[match[2]]);
          ongoing = '';
        } else if (typeof data[match[2]] === 'string') {
          ongoing += data[match[2]];
        }
        lastIndex = match.index + match[0].length;
      }

      if (lastIndex < message.length) {
        ongoing += message.substring(lastIndex);
      }

      if (ongoing.length) {
        //            retval.push(React.createElement(Text, null, ongoing));
        retval.push(ongoing);
      }

      if (retval.length === 1) {
        return retval[0];
      }
      return retval.join('');
    }

    /**
     * Sets the active language
     *
     * @param {string} language The language code
     */

  }, {
    key: 'setLanguage',
    value: function setLanguage(language) {
      if (this.fetchTranslations(language)) {
        this.activeLanguage = language;
      }
    }

    /**
     * Sets the original language as active
     */

  }, {
    key: 'setOriginalLanguage',
    value: function setOriginalLanguage() {
      this.activeLanguage = this.config.source;
    }

    /**
     * Fetches translations for the given language
     *
     * @param {*} language
     *
     * @return {bool} Returns TRUE on success, FALSE on failure
     */

  }, {
    key: 'fetchTranslations',
    value: function fetchTranslations(language) {
      this.setTranslations({
        'Test sentence': 'Phrase de test'
      });
      return true;
    }

    /**
     * Sets the active translations
     *
     * @param {Object} translations
     */

  }, {
    key: 'setTranslations',
    value: function setTranslations(translations) {
      this.translations = translations;
    }

    /**
     * Substitutes the message with symbols
     *
     * @param {String} message
     */

  }, {
    key: 'debugMode',
    value: function debugMode(message) {
      return '-'.repeat(message.length);
    }

    /**
     * Formats an error message to be returned instead of the translation
     *
     * @todo Add a console warning ? an exception ?
     * @param {String} message The error message
     */

  }, {
    key: 'error',
    value: function error(message) {
      return '\u2049\uFE0F ' + message;
    }

    /**
     * Checks and substitute plural in a message
     *
     * @todo Error handling
     * @todo Multiple plural patterns
     * @todo Would checking message.indexOf('plural') before anything speed up things? benchmark
     *
     * @param {String} message The translated message
     * @param {*} data The bound data
     */

  }, {
    key: 'pluralize',
    value: function pluralize(message, data) {

      var plural = false;

      if (plural = message.match(/{(\w),plural,(.*?)}}/)) {
        var quantity = plural[1];
        var chunks = plural[2].split(/=(\d|\*){/);
        var pos = chunks.indexOf('' + data[quantity]);

        if (pos === -1) {
          pos = chunks.indexOf('*');
          message = message.replace(plural[0], chunks[pos + 1].replace('#', data[quantity]));
        } else {
          message = message.replace(plural[0], chunks[pos + 1].replace(/}\s*$/, ''));
        }
      }

      return message;
    }
  }]);

  return Translator;
}();

export default Translator;